/*
 * Copyright 2010 Red Hat, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.commonjava.emb.nexus;

import java.text.MessageFormat;

public class AutoNXException
    extends Exception
{

    private static final long serialVersionUID = 1L;

    private final Object[] params;

    private String formattedMessage;

    public AutoNXException( final String message, final Throwable cause )
    {
        super( message, cause );
        params = null;
    }

    public AutoNXException( final String message )
    {
        super( message );
        params = null;
    }

    public AutoNXException( final String message, final Throwable cause, final Object... params )
    {
        super( message, cause );
        this.params = params;
    }

    public AutoNXException( final String message, final Object... params )
    {
        super( message );
        this.params = params;
    }

    @Override
    public synchronized String getMessage()
    {
        if ( formattedMessage == null )
        {
            final String format = super.getMessage();
            if ( params == null || params.length < 1 )
            {
                formattedMessage = format;
            }
            else
            {
                try
                {
                    formattedMessage = MessageFormat.format( format, params );
                }
                catch ( final Error e )
                {
                    formattedMessage = format;
                    throw e;
                }
                catch ( final RuntimeException e )
                {
                    formattedMessage = format;
                    throw e;
                }
                catch ( final Exception e )
                {
                    formattedMessage = format;
                }
            }
        }

        return formattedMessage;
    }

}
